﻿figures2d = new function () {
    /*canvas context variables*/
    var __context;
    var defaultHeight = figuresConfig.default2dHeight;
    var defaultWidth = figuresConfig.default2dWidth;
    var currentHeight = defaultHeight;
    var currentWidth = defaultWidth + (35 * 3 / 2);

    /*figures defaults*/
    var __figure_margin = currentWidth / 20;
    var __axis_numbers_margin = -(__figure_margin / 4);
    var __axis_numbers_size = 15;
    var __axis_numbers_style = __axis_numbers_size.toString() + "pt Arial";
    var __figure_bg_color = "#aaaaaa";
    var __figure_grid_bg_color = "#FFFFFF";
    var __axes_color = "#000000";
    var __plot_color = "#0000FF";

    /*private functions*/
    function __drawInTemporaryState(nested_drawing_function) {
        __context.save();
        nested_drawing_function();
        __context.restore();
    }

    function __getContext(canvas_id) {
        var canvas = $('#' + canvas_id);
        __context = canvas[0].getContext("2d");
        currentHeight = canvas.height();
        currentWidth = canvas.width();
        canvas[0].height = currentHeight;
        canvas[0].width = currentWidth;
        __figure_margin = __axis_numbers_size * 2;
        __axis_numbers_margin = -(__figure_margin / 4);
    }

    function __drawFigureBG() {
        __context.fillStyle = __figure_bg_color;
        __context.fillRect(0, 0, currentWidth, currentHeight);
    }

    function __drawGridBG() {
        __drawInTemporaryState(function () {
            __context.translate(__figure_margin, __figure_margin);
            __context.fillStyle = __figure_grid_bg_color;
            var margin = 2 * __figure_margin;
            __context.fillRect(0, 0, currentWidth - margin, currentHeight - margin);
        });
    }

    function __drawAxes(plot_data) {
        __drawInTemporaryState(function () {
            __context.strokeStyle = __axes_color;
            __context.translate(__figure_margin, __figure_margin);

            var margin = 2 * __figure_margin;
            __context.strokeRect(0 - 0.5, 0 - 0.5, currentWidth - margin + 1, currentHeight - margin + 1);
            __drawAxisNumbers(plot_data);

            function __drawAxisNumbers() {
                __setNormalTransform();
                __context.translate(__figure_margin, __figure_margin);
                __context.fillStyle = __axes_color;
                __context.font = __axis_numbers_style;
                __context.textAlign = 'center';
                __context.fillText(plot_data.XAxis[0], 0, __axis_numbers_margin);
                __context.fillText(plot_data.XAxis[1], currentWidth - __figure_margin * 2, __axis_numbers_margin);
                __context.textBaseline = 'middle';
                __context.fillText(plot_data.YAxis[0], __axis_numbers_margin * 2, currentHeight - 2 * __figure_margin);
                __context.fillText(plot_data.YAxis[1], __axis_numbers_margin * 2, 0);
            }
        });
    }

    function __drawPlotPoints(plot_data) {
        __drawInTemporaryState(function () {

            function __translatePlotPoint(x, y) {
                var double_margin = 2 * __figure_margin;
                var width_plot_space = currentWidth - double_margin;
                var height_plot_space = currentHeight - double_margin;
                var x_in_relative_to_axis = ((x - XAxis_min) / (XAxis[1] - XAxis_min)) * width_plot_space;
                var y_in_relative_to_axis = ((y - YAxis_min) / (YAxis[1] - YAxis_min)) * height_plot_space;
                return { x: x_in_relative_to_axis, y: y_in_relative_to_axis };
            }

            function __findMaxValues(x_points, y_points) {
                return { x: Math.max.apply(Math, x_points), y: Math.max.apply(Math, y_points) };
            }

            function __findMinValues(x_points, y_points) {
                return { x: Math.min.apply(Math, x_points), y: Math.min.apply(Math, y_points) };
            }

            var x_points = plot_data.XData;
            var y_points = plot_data.YData;
            var XAxis = plot_data.XAxis;
            var YAxis = plot_data.YAxis;
            var XAxis_min = XAxis[0];
            var YAxis_min = YAxis[0];
            var XAxis_space = XAxis[1] - XAxis_min;
            var YAxis_space = YAxis[1] - YAxis_min;

            __context.strokeStyle = __plot_color;
            __context.translate(__figure_margin, __figure_margin);
            __context.beginPath();
            var translated_point = __translatePlotPoint(x_points[0], y_points[0]);
            __context.lineTo(translated_point.x, translated_point.y);
            for (var i = 1; i < x_points.length; ++i) {
                translated_point = __translatePlotPoint(x_points[i], y_points[i]);
                __context.lineTo(translated_point.x, translated_point.y);
            }
            __context.stroke();
        });
    }

    function __drawSinglePoint(x, y) {
        __drawInTemporaryState(function () {
            __context.strokeStyle = __plot_color;
            __context.translate(currentWidth / 2, currentHeight / 2);
            __context.beginPath();
            __context.moveTo(0, 0);
            __context.lineTo(1, 1);
            __context.stroke();
        });
    }

    function __setCartezianTransform() {
        __context.setTransform(1, 0, 0, -1, 0, currentHeight);
    }

    function __setNormalTransform() {
        __context.setTransform(1, 0, 0, 1, 0, 0);
    }

    function addFigureCanvas(canvas_id, handle) {
        figuresDrawer.getFiguresContainer().append(
            '<canvas id="' + canvas_id +
            '" width="' + currentWidth + '" height="' + currentHeight +
            '" title="Figure ' + handle + '"></canvas>');
    };

    function openDialog(canvasId, plot_data, handle) {
        $('#' + canvasId).dialog({
            position: [handle * defaultHeight, 0],
            width: defaultWidth,
            height: defaultHeight,
            resize: function (event, ui) {
                id = event.target.id;
                deferedExec(function () {
                    figures2d.drawOne(id, plot_data);
                }, 2000);
            }
        });
    };

    /*public functions*/
    this.drawOne = function (canvasId, plot_data) {
        var start = new Date();
        if ($('#' + canvasId).length == 0) {
            var handle = canvasId.split(figuresConfig.partialCanvasId)[1];
            addFigureCanvas(canvasId, handle);
            openDialog(canvasId, plot_data, handle);
        }
        __getContext(canvasId);
        __setCartezianTransform();
        __drawFigureBG();
        __drawGridBG();
        __drawAxes(plot_data);
        if (plot_data.XData.length > 1) {
            __drawPlotPoints(plot_data);
        }
        else __drawSinglePoint();
        var end = new Date;
    };
} ();